# -----------------------------------------------------------------
# ** ATTENTION **
# This code was automatically generated from _c_ast.cfg
#
# Do not modify it directly. Modify the configuration file and
# run the generator again.
# ** ** *** ** **
#
# pycparser: c_ast.py
#
# AST Node classes.
#
# Eli Bendersky [https://eli.thegreenplace.net/]
# License: BSD
# -----------------------------------------------------------------


import sys
from typing import Any, ClassVar, IO, Optional


def _repr(obj):
    """
    Get the representation of an object, with dedicated pprint-like format for lists.
    """
    if isinstance(obj, list):
        return "[" + (",\n ".join((_repr(e).replace("\n", "\n ") for e in obj))) + "\n]"
    else:
        return repr(obj)


class Node:
    __slots__ = ()
    """ Abstract base class for AST nodes.
    """
    attr_names: ClassVar[tuple[str, ...]] = ()
    coord: Optional[Any]

    def __repr__(self):
        """Generates a python representation of the current node"""
        result = self.__class__.__name__ + "("

        indent = ""
        separator = ""
        for name in self.__slots__[:-2]:
            result += separator
            result += indent
            result += (
                name
                + "="
                + (
                    _repr(getattr(self, name)).replace(
                        "\n",
                        "\n  " + (" " * (len(name) + len(self.__class__.__name__))),
                    )
                )
            )

            separator = ","
            indent = "\n " + (" " * len(self.__class__.__name__))

        result += indent + ")"

        return result

    def children(self):
        """A sequence of all children that are Nodes"""
        pass

    def show(
        self,
        buf: IO[str] = sys.stdout,
        offset: int = 0,
        attrnames: bool = False,
        showemptyattrs: bool = True,
        nodenames: bool = False,
        showcoord: bool = False,
        _my_node_name: Optional[str] = None,
    ):
        """Pretty print the Node and all its attributes and
        children (recursively) to a buffer.

        buf:
            Open IO buffer into which the Node is printed.

        offset:
            Initial offset (amount of leading spaces)

        attrnames:
            True if you want to see the attribute names in
            name=value pairs. False to only see the values.

        showemptyattrs:
            False if you want to suppress printing empty attributes.

        nodenames:
            True if you want to see the actual node names
            within their parents.

        showcoord:
            Do you want the coordinates of each Node to be
            displayed.
        """
        lead = " " * offset
        if nodenames and _my_node_name is not None:
            buf.write(lead + self.__class__.__name__ + " <" + _my_node_name + ">: ")
        else:
            buf.write(lead + self.__class__.__name__ + ": ")

        if self.attr_names:

            def is_empty(v):
                v is None or (hasattr(v, "__len__") and len(v) == 0)

            nvlist = [
                (n, getattr(self, n))
                for n in self.attr_names
                if showemptyattrs or not is_empty(getattr(self, n))
            ]
            if attrnames:
                attrstr = ", ".join(f"{name}={value}" for name, value in nvlist)
            else:
                attrstr = ", ".join(f"{value}" for _, value in nvlist)
            buf.write(attrstr)

        if showcoord:
            buf.write(f" (at {self.coord})")
        buf.write("\n")

        for child_name, child in self.children():
            child.show(
                buf,
                offset=offset + 2,
                attrnames=attrnames,
                showemptyattrs=showemptyattrs,
                nodenames=nodenames,
                showcoord=showcoord,
                _my_node_name=child_name,
            )


class NodeVisitor:
    """A base NodeVisitor class for visiting c_ast nodes.
    Subclass it and define your own visit_XXX methods, where
    XXX is the class name you want to visit with these
    methods.

    For example:

    class ConstantVisitor(NodeVisitor):
        def __init__(self):
            self.values = []

        def visit_Constant(self, node):
            self.values.append(node.value)

    Creates a list of values of all the constant nodes
    encountered below the given node. To use it:

    cv = ConstantVisitor()
    cv.visit(node)

    Notes:

    *   generic_visit() will be called for AST nodes for which
        no visit_XXX method was defined.
    *   The children of nodes for which a visit_XXX was
        defined will not be visited - if you need this, call
        generic_visit() on the node.
        You can use:
            NodeVisitor.generic_visit(self, node)
    *   Modeled after Python's own AST visiting facilities
        (the ast module of Python 3.0)
    """

    _method_cache = None

    def visit(self, node: Node):
        """Visit a node."""

        if self._method_cache is None:
            self._method_cache = {}

        visitor = self._method_cache.get(node.__class__.__name__, None)
        if visitor is None:
            method = "visit_" + node.__class__.__name__
            visitor = getattr(self, method, self.generic_visit)
            self._method_cache[node.__class__.__name__] = visitor

        return visitor(node)

    def generic_visit(self, node: Node):
        """Called if no explicit visitor function exists for a
        node. Implements preorder visiting of the node.
        """
        for _, c in node.children():
            self.visit(c)


class ArrayDecl(Node):
    __slots__ = ("type", "dim", "dim_quals", "coord", "__weakref__")

    def __init__(self, type, dim, dim_quals, coord=None):
        self.type = type
        self.dim = dim
        self.dim_quals = dim_quals
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        if self.dim is not None:
            nodelist.append(("dim", self.dim))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type
        if self.dim is not None:
            yield self.dim

    attr_names = ("dim_quals",)


class ArrayRef(Node):
    __slots__ = ("name", "subscript", "coord", "__weakref__")

    def __init__(self, name, subscript, coord=None):
        self.name = name
        self.subscript = subscript
        self.coord = coord

    def children(self):
        nodelist = []
        if self.name is not None:
            nodelist.append(("name", self.name))
        if self.subscript is not None:
            nodelist.append(("subscript", self.subscript))
        return tuple(nodelist)

    def __iter__(self):
        if self.name is not None:
            yield self.name
        if self.subscript is not None:
            yield self.subscript

    attr_names = ()


class Assignment(Node):
    __slots__ = ("op", "lvalue", "rvalue", "coord", "__weakref__")

    def __init__(self, op, lvalue, rvalue, coord=None):
        self.op = op
        self.lvalue = lvalue
        self.rvalue = rvalue
        self.coord = coord

    def children(self):
        nodelist = []
        if self.lvalue is not None:
            nodelist.append(("lvalue", self.lvalue))
        if self.rvalue is not None:
            nodelist.append(("rvalue", self.rvalue))
        return tuple(nodelist)

    def __iter__(self):
        if self.lvalue is not None:
            yield self.lvalue
        if self.rvalue is not None:
            yield self.rvalue

    attr_names = ("op",)


class Alignas(Node):
    __slots__ = ("alignment", "coord", "__weakref__")

    def __init__(self, alignment, coord=None):
        self.alignment = alignment
        self.coord = coord

    def children(self):
        nodelist = []
        if self.alignment is not None:
            nodelist.append(("alignment", self.alignment))
        return tuple(nodelist)

    def __iter__(self):
        if self.alignment is not None:
            yield self.alignment

    attr_names = ()


class BinaryOp(Node):
    __slots__ = ("op", "left", "right", "coord", "__weakref__")

    def __init__(self, op, left, right, coord=None):
        self.op = op
        self.left = left
        self.right = right
        self.coord = coord

    def children(self):
        nodelist = []
        if self.left is not None:
            nodelist.append(("left", self.left))
        if self.right is not None:
            nodelist.append(("right", self.right))
        return tuple(nodelist)

    def __iter__(self):
        if self.left is not None:
            yield self.left
        if self.right is not None:
            yield self.right

    attr_names = ("op",)


class Break(Node):
    __slots__ = ("coord", "__weakref__")

    def __init__(self, coord=None):
        self.coord = coord

    def children(self):
        return ()

    def __iter__(self):
        return
        yield

    attr_names = ()


class Case(Node):
    __slots__ = ("expr", "stmts", "coord", "__weakref__")

    def __init__(self, expr, stmts, coord=None):
        self.expr = expr
        self.stmts = stmts
        self.coord = coord

    def children(self):
        nodelist = []
        if self.expr is not None:
            nodelist.append(("expr", self.expr))
        for i, child in enumerate(self.stmts or []):
            nodelist.append((f"stmts[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        if self.expr is not None:
            yield self.expr
        for child in self.stmts or []:
            yield child

    attr_names = ()


class Cast(Node):
    __slots__ = ("to_type", "expr", "coord", "__weakref__")

    def __init__(self, to_type, expr, coord=None):
        self.to_type = to_type
        self.expr = expr
        self.coord = coord

    def children(self):
        nodelist = []
        if self.to_type is not None:
            nodelist.append(("to_type", self.to_type))
        if self.expr is not None:
            nodelist.append(("expr", self.expr))
        return tuple(nodelist)

    def __iter__(self):
        if self.to_type is not None:
            yield self.to_type
        if self.expr is not None:
            yield self.expr

    attr_names = ()


class Compound(Node):
    __slots__ = ("block_items", "coord", "__weakref__")

    def __init__(self, block_items, coord=None):
        self.block_items = block_items
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.block_items or []):
            nodelist.append((f"block_items[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.block_items or []:
            yield child

    attr_names = ()


class CompoundLiteral(Node):
    __slots__ = ("type", "init", "coord", "__weakref__")

    def __init__(self, type, init, coord=None):
        self.type = type
        self.init = init
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        if self.init is not None:
            nodelist.append(("init", self.init))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type
        if self.init is not None:
            yield self.init

    attr_names = ()


class Constant(Node):
    __slots__ = ("type", "value", "coord", "__weakref__")

    def __init__(self, type, value, coord=None):
        self.type = type
        self.value = value
        self.coord = coord

    def children(self):
        nodelist = []
        return tuple(nodelist)

    def __iter__(self):
        return
        yield

    attr_names = (
        "type",
        "value",
    )


class Continue(Node):
    __slots__ = ("coord", "__weakref__")

    def __init__(self, coord=None):
        self.coord = coord

    def children(self):
        return ()

    def __iter__(self):
        return
        yield

    attr_names = ()


class Decl(Node):
    __slots__ = (
        "name",
        "quals",
        "align",
        "storage",
        "funcspec",
        "type",
        "init",
        "bitsize",
        "coord",
        "__weakref__",
    )

    def __init__(
        self, name, quals, align, storage, funcspec, type, init, bitsize, coord=None
    ):
        self.name = name
        self.quals = quals
        self.align = align
        self.storage = storage
        self.funcspec = funcspec
        self.type = type
        self.init = init
        self.bitsize = bitsize
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        if self.init is not None:
            nodelist.append(("init", self.init))
        if self.bitsize is not None:
            nodelist.append(("bitsize", self.bitsize))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type
        if self.init is not None:
            yield self.init
        if self.bitsize is not None:
            yield self.bitsize

    attr_names = (
        "name",
        "quals",
        "align",
        "storage",
        "funcspec",
    )


class DeclList(Node):
    __slots__ = ("decls", "coord", "__weakref__")

    def __init__(self, decls, coord=None):
        self.decls = decls
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.decls or []):
            nodelist.append((f"decls[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.decls or []:
            yield child

    attr_names = ()


class Default(Node):
    __slots__ = ("stmts", "coord", "__weakref__")

    def __init__(self, stmts, coord=None):
        self.stmts = stmts
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.stmts or []):
            nodelist.append((f"stmts[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.stmts or []:
            yield child

    attr_names = ()


class DoWhile(Node):
    __slots__ = ("cond", "stmt", "coord", "__weakref__")

    def __init__(self, cond, stmt, coord=None):
        self.cond = cond
        self.stmt = stmt
        self.coord = coord

    def children(self):
        nodelist = []
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.stmt is not None:
            nodelist.append(("stmt", self.stmt))
        return tuple(nodelist)

    def __iter__(self):
        if self.cond is not None:
            yield self.cond
        if self.stmt is not None:
            yield self.stmt

    attr_names = ()


class EllipsisParam(Node):
    __slots__ = ("coord", "__weakref__")

    def __init__(self, coord=None):
        self.coord = coord

    def children(self):
        return ()

    def __iter__(self):
        return
        yield

    attr_names = ()


class EmptyStatement(Node):
    __slots__ = ("coord", "__weakref__")

    def __init__(self, coord=None):
        self.coord = coord

    def children(self):
        return ()

    def __iter__(self):
        return
        yield

    attr_names = ()


class Enum(Node):
    __slots__ = ("name", "values", "coord", "__weakref__")

    def __init__(self, name, values, coord=None):
        self.name = name
        self.values = values
        self.coord = coord

    def children(self):
        nodelist = []
        if self.values is not None:
            nodelist.append(("values", self.values))
        return tuple(nodelist)

    def __iter__(self):
        if self.values is not None:
            yield self.values

    attr_names = ("name",)


class Enumerator(Node):
    __slots__ = ("name", "value", "coord", "__weakref__")

    def __init__(self, name, value, coord=None):
        self.name = name
        self.value = value
        self.coord = coord

    def children(self):
        nodelist = []
        if self.value is not None:
            nodelist.append(("value", self.value))
        return tuple(nodelist)

    def __iter__(self):
        if self.value is not None:
            yield self.value

    attr_names = ("name",)


class EnumeratorList(Node):
    __slots__ = ("enumerators", "coord", "__weakref__")

    def __init__(self, enumerators, coord=None):
        self.enumerators = enumerators
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.enumerators or []):
            nodelist.append((f"enumerators[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.enumerators or []:
            yield child

    attr_names = ()


class ExprList(Node):
    __slots__ = ("exprs", "coord", "__weakref__")

    def __init__(self, exprs, coord=None):
        self.exprs = exprs
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.exprs or []):
            nodelist.append((f"exprs[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.exprs or []:
            yield child

    attr_names = ()


class FileAST(Node):
    __slots__ = ("ext", "coord", "__weakref__")

    def __init__(self, ext, coord=None):
        self.ext = ext
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.ext or []):
            nodelist.append((f"ext[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.ext or []:
            yield child

    attr_names = ()


class For(Node):
    __slots__ = ("init", "cond", "next", "stmt", "coord", "__weakref__")

    def __init__(self, init, cond, next, stmt, coord=None):
        self.init = init
        self.cond = cond
        self.next = next
        self.stmt = stmt
        self.coord = coord

    def children(self):
        nodelist = []
        if self.init is not None:
            nodelist.append(("init", self.init))
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.next is not None:
            nodelist.append(("next", self.next))
        if self.stmt is not None:
            nodelist.append(("stmt", self.stmt))
        return tuple(nodelist)

    def __iter__(self):
        if self.init is not None:
            yield self.init
        if self.cond is not None:
            yield self.cond
        if self.next is not None:
            yield self.next
        if self.stmt is not None:
            yield self.stmt

    attr_names = ()


class FuncCall(Node):
    __slots__ = ("name", "args", "coord", "__weakref__")

    def __init__(self, name, args, coord=None):
        self.name = name
        self.args = args
        self.coord = coord

    def children(self):
        nodelist = []
        if self.name is not None:
            nodelist.append(("name", self.name))
        if self.args is not None:
            nodelist.append(("args", self.args))
        return tuple(nodelist)

    def __iter__(self):
        if self.name is not None:
            yield self.name
        if self.args is not None:
            yield self.args

    attr_names = ()


class FuncDecl(Node):
    __slots__ = ("args", "type", "coord", "__weakref__")

    def __init__(self, args, type, coord=None):
        self.args = args
        self.type = type
        self.coord = coord

    def children(self):
        nodelist = []
        if self.args is not None:
            nodelist.append(("args", self.args))
        if self.type is not None:
            nodelist.append(("type", self.type))
        return tuple(nodelist)

    def __iter__(self):
        if self.args is not None:
            yield self.args
        if self.type is not None:
            yield self.type

    attr_names = ()


class FuncDef(Node):
    __slots__ = ("decl", "param_decls", "body", "coord", "__weakref__")

    def __init__(self, decl, param_decls, body, coord=None):
        self.decl = decl
        self.param_decls = param_decls
        self.body = body
        self.coord = coord

    def children(self):
        nodelist = []
        if self.decl is not None:
            nodelist.append(("decl", self.decl))
        if self.body is not None:
            nodelist.append(("body", self.body))
        for i, child in enumerate(self.param_decls or []):
            nodelist.append((f"param_decls[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        if self.decl is not None:
            yield self.decl
        if self.body is not None:
            yield self.body
        for child in self.param_decls or []:
            yield child

    attr_names = ()


class Goto(Node):
    __slots__ = ("name", "coord", "__weakref__")

    def __init__(self, name, coord=None):
        self.name = name
        self.coord = coord

    def children(self):
        nodelist = []
        return tuple(nodelist)

    def __iter__(self):
        return
        yield

    attr_names = ("name",)


class ID(Node):
    __slots__ = ("name", "coord", "__weakref__")

    def __init__(self, name, coord=None):
        self.name = name
        self.coord = coord

    def children(self):
        nodelist = []
        return tuple(nodelist)

    def __iter__(self):
        return
        yield

    attr_names = ("name",)


class IdentifierType(Node):
    __slots__ = ("names", "coord", "__weakref__")

    def __init__(self, names, coord=None):
        self.names = names
        self.coord = coord

    def children(self):
        nodelist = []
        return tuple(nodelist)

    def __iter__(self):
        return
        yield

    attr_names = ("names",)


class If(Node):
    __slots__ = ("cond", "iftrue", "iffalse", "coord", "__weakref__")

    def __init__(self, cond, iftrue, iffalse, coord=None):
        self.cond = cond
        self.iftrue = iftrue
        self.iffalse = iffalse
        self.coord = coord

    def children(self):
        nodelist = []
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.iftrue is not None:
            nodelist.append(("iftrue", self.iftrue))
        if self.iffalse is not None:
            nodelist.append(("iffalse", self.iffalse))
        return tuple(nodelist)

    def __iter__(self):
        if self.cond is not None:
            yield self.cond
        if self.iftrue is not None:
            yield self.iftrue
        if self.iffalse is not None:
            yield self.iffalse

    attr_names = ()


class InitList(Node):
    __slots__ = ("exprs", "coord", "__weakref__")

    def __init__(self, exprs, coord=None):
        self.exprs = exprs
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.exprs or []):
            nodelist.append((f"exprs[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.exprs or []:
            yield child

    attr_names = ()


class Label(Node):
    __slots__ = ("name", "stmt", "coord", "__weakref__")

    def __init__(self, name, stmt, coord=None):
        self.name = name
        self.stmt = stmt
        self.coord = coord

    def children(self):
        nodelist = []
        if self.stmt is not None:
            nodelist.append(("stmt", self.stmt))
        return tuple(nodelist)

    def __iter__(self):
        if self.stmt is not None:
            yield self.stmt

    attr_names = ("name",)


class NamedInitializer(Node):
    __slots__ = ("name", "expr", "coord", "__weakref__")

    def __init__(self, name, expr, coord=None):
        self.name = name
        self.expr = expr
        self.coord = coord

    def children(self):
        nodelist = []
        if self.expr is not None:
            nodelist.append(("expr", self.expr))
        for i, child in enumerate(self.name or []):
            nodelist.append((f"name[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        if self.expr is not None:
            yield self.expr
        for child in self.name or []:
            yield child

    attr_names = ()


class ParamList(Node):
    __slots__ = ("params", "coord", "__weakref__")

    def __init__(self, params, coord=None):
        self.params = params
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.params or []):
            nodelist.append((f"params[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.params or []:
            yield child

    attr_names = ()


class PtrDecl(Node):
    __slots__ = ("quals", "type", "coord", "__weakref__")

    def __init__(self, quals, type, coord=None):
        self.quals = quals
        self.type = type
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type

    attr_names = ("quals",)


class Return(Node):
    __slots__ = ("expr", "coord", "__weakref__")

    def __init__(self, expr, coord=None):
        self.expr = expr
        self.coord = coord

    def children(self):
        nodelist = []
        if self.expr is not None:
            nodelist.append(("expr", self.expr))
        return tuple(nodelist)

    def __iter__(self):
        if self.expr is not None:
            yield self.expr

    attr_names = ()


class StaticAssert(Node):
    __slots__ = ("cond", "message", "coord", "__weakref__")

    def __init__(self, cond, message, coord=None):
        self.cond = cond
        self.message = message
        self.coord = coord

    def children(self):
        nodelist = []
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.message is not None:
            nodelist.append(("message", self.message))
        return tuple(nodelist)

    def __iter__(self):
        if self.cond is not None:
            yield self.cond
        if self.message is not None:
            yield self.message

    attr_names = ()


class Struct(Node):
    __slots__ = ("name", "decls", "coord", "__weakref__")

    def __init__(self, name, decls, coord=None):
        self.name = name
        self.decls = decls
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.decls or []):
            nodelist.append((f"decls[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.decls or []:
            yield child

    attr_names = ("name",)


class StructRef(Node):
    __slots__ = ("name", "type", "field", "coord", "__weakref__")

    def __init__(self, name, type, field, coord=None):
        self.name = name
        self.type = type
        self.field = field
        self.coord = coord

    def children(self):
        nodelist = []
        if self.name is not None:
            nodelist.append(("name", self.name))
        if self.field is not None:
            nodelist.append(("field", self.field))
        return tuple(nodelist)

    def __iter__(self):
        if self.name is not None:
            yield self.name
        if self.field is not None:
            yield self.field

    attr_names = ("type",)


class Switch(Node):
    __slots__ = ("cond", "stmt", "coord", "__weakref__")

    def __init__(self, cond, stmt, coord=None):
        self.cond = cond
        self.stmt = stmt
        self.coord = coord

    def children(self):
        nodelist = []
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.stmt is not None:
            nodelist.append(("stmt", self.stmt))
        return tuple(nodelist)

    def __iter__(self):
        if self.cond is not None:
            yield self.cond
        if self.stmt is not None:
            yield self.stmt

    attr_names = ()


class TernaryOp(Node):
    __slots__ = ("cond", "iftrue", "iffalse", "coord", "__weakref__")

    def __init__(self, cond, iftrue, iffalse, coord=None):
        self.cond = cond
        self.iftrue = iftrue
        self.iffalse = iffalse
        self.coord = coord

    def children(self):
        nodelist = []
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.iftrue is not None:
            nodelist.append(("iftrue", self.iftrue))
        if self.iffalse is not None:
            nodelist.append(("iffalse", self.iffalse))
        return tuple(nodelist)

    def __iter__(self):
        if self.cond is not None:
            yield self.cond
        if self.iftrue is not None:
            yield self.iftrue
        if self.iffalse is not None:
            yield self.iffalse

    attr_names = ()


class TypeDecl(Node):
    __slots__ = ("declname", "quals", "align", "type", "coord", "__weakref__")

    def __init__(self, declname, quals, align, type, coord=None):
        self.declname = declname
        self.quals = quals
        self.align = align
        self.type = type
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type

    attr_names = (
        "declname",
        "quals",
        "align",
    )


class Typedef(Node):
    __slots__ = ("name", "quals", "storage", "type", "coord", "__weakref__")

    def __init__(self, name, quals, storage, type, coord=None):
        self.name = name
        self.quals = quals
        self.storage = storage
        self.type = type
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type

    attr_names = (
        "name",
        "quals",
        "storage",
    )


class Typename(Node):
    __slots__ = ("name", "quals", "align", "type", "coord", "__weakref__")

    def __init__(self, name, quals, align, type, coord=None):
        self.name = name
        self.quals = quals
        self.align = align
        self.type = type
        self.coord = coord

    def children(self):
        nodelist = []
        if self.type is not None:
            nodelist.append(("type", self.type))
        return tuple(nodelist)

    def __iter__(self):
        if self.type is not None:
            yield self.type

    attr_names = (
        "name",
        "quals",
        "align",
    )


class UnaryOp(Node):
    __slots__ = ("op", "expr", "coord", "__weakref__")

    def __init__(self, op, expr, coord=None):
        self.op = op
        self.expr = expr
        self.coord = coord

    def children(self):
        nodelist = []
        if self.expr is not None:
            nodelist.append(("expr", self.expr))
        return tuple(nodelist)

    def __iter__(self):
        if self.expr is not None:
            yield self.expr

    attr_names = ("op",)


class Union(Node):
    __slots__ = ("name", "decls", "coord", "__weakref__")

    def __init__(self, name, decls, coord=None):
        self.name = name
        self.decls = decls
        self.coord = coord

    def children(self):
        nodelist = []
        for i, child in enumerate(self.decls or []):
            nodelist.append((f"decls[{i}]", child))
        return tuple(nodelist)

    def __iter__(self):
        for child in self.decls or []:
            yield child

    attr_names = ("name",)


class While(Node):
    __slots__ = ("cond", "stmt", "coord", "__weakref__")

    def __init__(self, cond, stmt, coord=None):
        self.cond = cond
        self.stmt = stmt
        self.coord = coord

    def children(self):
        nodelist = []
        if self.cond is not None:
            nodelist.append(("cond", self.cond))
        if self.stmt is not None:
            nodelist.append(("stmt", self.stmt))
        return tuple(nodelist)

    def __iter__(self):
        if self.cond is not None:
            yield self.cond
        if self.stmt is not None:
            yield self.stmt

    attr_names = ()


class Pragma(Node):
    __slots__ = ("string", "coord", "__weakref__")

    def __init__(self, string, coord=None):
        self.string = string
        self.coord = coord

    def children(self):
        nodelist = []
        return tuple(nodelist)

    def __iter__(self):
        return
        yield

    attr_names = ("string",)



from __future__ import annotations

import asyncio
import contextvars
import logging
import sys
from collections.abc import Callable, Generator
from typing import Any, Literal
from urllib.parse import unquote

import zttp

from uvicorn._types import (
    ASGI3Application,
    ASGIReceiveEvent,
    ASGISendEvent,
    HTTPRequestEvent,
    HTTPResponseBodyEvent,
    HTTPResponseStartEvent,
    HTTPScope,
)
from uvicorn.config import Config
from uvicorn.logging import TRACE_LOG_LEVEL
from uvicorn.protocols.http.flow_control import CLOSE_HEADER, HIGH_WATER_LIMIT, FlowControl, service_unavailable
from uvicorn.protocols.utils import get_client_addr, get_local_addr, get_path_with_query_string, get_remote_addr, is_ssl
from uvicorn.server import ServerState


class ZttpProtocol(asyncio.Protocol):
    def __init__(
        self,
        config: Config,
        server_state: ServerState,
        app_state: dict[str, Any],
        _loop: asyncio.AbstractEventLoop | None = None,
    ) -> None:
        if not config.loaded:
            config.load()

        self.config = config
        self.app = config.loaded_app
        self.loop = _loop or asyncio.get_event_loop()
        self.logger = logging.getLogger("uvicorn.error")
        self.access_logger = logging.getLogger("uvicorn.access")
        self.access_log = self.access_logger.hasHandlers()
        self.conn = zttp.Connection(zttp.SERVER)
        self.ws_protocol_class = config.ws_protocol_class
        self.root_path = config.root_path
        self.asgi_version = config.asgi_version
        self.limit_concurrency = config.limit_concurrency
        self.app_state = app_state

        # Timeouts
        self.timeout_keep_alive_task: asyncio.TimerHandle | None = None
        self.timeout_keep_alive = config.timeout_keep_alive

        # Shared server state
        self.server_state = server_state
        self.connections = server_state.connections
        self.tasks = server_state.tasks

        # Per-connection state
        self.transport: asyncio.Transport = None  # type: ignore[assignment]
        self.flow: FlowControl = None  # type: ignore[assignment]
        self.server: tuple[str, int | None] | None = None
        self.client: tuple[str, int] | None = None
        self.scheme: Literal["http", "https"] | None = None

        # Per-request state
        self.scope: HTTPScope = None  # type: ignore[assignment]
        self.headers: list[tuple[bytes, bytes]] = None  # type: ignore[assignment]
        self.cycle: RequestResponseCycle = None  # type: ignore[assignment]

    # Protocol interface
    def connection_made(  # type: ignore[override]
        self, transport: asyncio.Transport
    ) -> None:
        self.connections.add(self)

        self.transport = transport
        self.flow = FlowControl(transport)
        self.server = get_local_addr(transport)
        self.client = get_remote_addr(transport)
        self.scheme = "https" if is_ssl(transport) else "http"

        if self.logger.level <= TRACE_LOG_LEVEL:
            prefix = "%s:%d - " % self.client if self.client else ""
            self.logger.log(TRACE_LOG_LEVEL, "%sHTTP connection made", prefix)

    def connection_lost(self, exc: Exception | None) -> None:
        self.connections.discard(self)

        if self.logger.level <= TRACE_LOG_LEVEL:
            prefix = "%s:%d - " % self.client if self.client else ""
            self.logger.log(TRACE_LOG_LEVEL, "%sHTTP connection lost", prefix)

        if self.cycle and not self.cycle.response_complete:
            self.cycle.disconnected = True
        if self.cycle is not None:
            self.cycle.message_event.set()
        if self.flow is not None:
            self.flow.resume_writing()
        if exc is None:
            self.transport.close()
            self._unset_keepalive_if_required()

    def eof_received(self) -> None:
        pass

    def _unset_keepalive_if_required(self) -> None:
        if self.timeout_keep_alive_task is not None:
            self.timeout_keep_alive_task.cancel()
            self.timeout_keep_alive_task = None

    def _should_upgrade_to_ws(self) -> bool:
        if self.ws_protocol_class is None:
            return False
        return True

    def _unsupported_upgrade_warning(self) -> None:
        self.logger.warning("Unsupported upgrade request.")
        if not self._should_upgrade_to_ws():
            self.logger.warning(
                "No supported WebSocket library detected. "
                "Please use \"pip install 'uvicorn[standard]'\", or install 'websockets' or 'wsproto' manually."
            )

    def _should_upgrade(self) -> bool:
        upgrade = self.conn.upgrade()
        if upgrade is not None:
            upgrade = upgrade.lower()
        if upgrade == b"websocket" and self._should_upgrade_to_ws():
            return True
        if upgrade is not None:
            self._unsupported_upgrade_warning()
        return False

    def data_received(self, data: bytes) -> None:
        self._unset_keepalive_if_required()

        self.conn.receive_data(data)
        self.handle_events()

    def events(self) -> Generator[zttp.Event]:
        """Yield every complete event currently available."""
        while True:
            event = self.conn.next_event()
            if event is zttp.NEED_DATA:
                return
            yield event

    def handle_events(self) -> None:
        try:
            for event in self.events():
                if isinstance(event, zttp.Request):
                    self.headers = [(key.lower(), value) for key, value in event.headers]
                    path = unquote(event.path.decode("ascii"))
                    full_path = self.root_path + path
                    full_raw_path = self.root_path.encode("ascii") + event.path
                    self.scope = {
                        "type": "http",
                        "asgi": {"version": self.asgi_version, "spec_version": "2.3"},
                        "http_version": event.http_version.decode("ascii"),
                        "server": self.server,
                        "client": self.client,
                        "scheme": self.scheme,  # type: ignore[typeddict-item]
                        "method": event.method.decode("ascii"),
                        "root_path": self.root_path,
                        "path": full_path,
                        "raw_path": full_raw_path,
                        "query_string": event.query,
                        "headers": self.headers,
                        "state": self.app_state.copy(),
                    }
                    if self._should_upgrade():
                        self.handle_websocket_upgrade(event)
                        return

                    # Handle 503 responses when 'limit_concurrency' is exceeded.
                    if self.limit_concurrency is not None and (
                        len(self.connections) >= self.limit_concurrency or len(self.tasks) >= self.limit_concurrency
                    ):
                        app = service_unavailable
                        message = "Exceeded concurrency limit."
                        self.logger.warning(message)
                    else:
                        app = self.app

                    self._unset_keepalive_if_required()

                    self.cycle = RequestResponseCycle(
                        scope=self.scope,
                        conn=self.conn,
                        transport=self.transport,
                        flow=self.flow,
                        logger=self.logger,
                        access_logger=self.access_logger,
                        access_log=self.access_log,
                        default_headers=self.server_state.default_headers,
                        message_event=asyncio.Event(),
                        expect_100_continue=event.expect_continue,
                        on_response=self.on_response_complete,
                    )
                    if self.config.reset_contextvars:
                        if sys.version_info >= (3, 11):  # pragma: py-lt-311
                            task = self.loop.create_task(self.cycle.run_asgi(app), context=contextvars.Context())
                        else:  # pragma: py-gte-311
                            task = contextvars.Context().run(self.loop.create_task, self.cycle.run_asgi(app))
                    else:
                        task = self.loop.create_task(self.cycle.run_asgi(app))
                    task.add_done_callback(self.tasks.discard)
                    self.tasks.add(task)

                elif isinstance(event, zttp.Data):
                    if self.cycle is None or self.cycle.response_complete:
                        continue  # pragma: no cover
                    self.cycle.body += event.data
                    if len(self.cycle.body) > HIGH_WATER_LIMIT:
                        self.flow.pause_reading()
                    self.cycle.message_event.set()

                elif isinstance(event, zttp.EndOfMessage):
                    if self.cycle is None:
                        continue  # pragma: no cover
                    self.cycle.more_body = False
                    self.cycle.message_event.set()
                    if self.cycle.response_complete:
                        self.conn.start_next_cycle()
        except zttp.RemoteProtocolError:
            msg = "Invalid HTTP request received."
            self.logger.warning(msg)
            self.send_400_response(msg)

    def handle_websocket_upgrade(self, event: zttp.Request) -> None:
        if self.logger.level <= TRACE_LOG_LEVEL:  # pragma: no cover
            prefix = "%s:%d - " % self.client if self.client else ""
            self.logger.log(TRACE_LOG_LEVEL, "%sUpgrading to WebSocket", prefix)

        self.connections.discard(self)
        output = bytearray(event.method + b" " + event.target + b" HTTP/1.1\r\n")
        for name, value in self.headers:
            output += name + b": " + value + b"\r\n"
        output += b"\r\n"
        protocol = self.ws_protocol_class(  # type: ignore[call-arg, misc]
            config=self.config,
            server_state=self.server_state,
            app_state=self.app_state,
        )
        protocol.connection_made(self.transport)
        protocol.data_received(bytes(output))
        self.transport.set_protocol(protocol)

    def send_400_response(self, msg: str) -> None:
        body = msg.encode("ascii")
        headers: list[tuple[bytes, bytes]] = [
            (b"content-type", b"text/plain; charset=utf-8"),
            (b"content-length", str(len(body)).encode("ascii")),
            (b"connection", b"close"),
        ]
        self.conn.send_response(400, headers)
        self.conn.send_data(body)
        self.conn.end_message()
        self.transport.write(self.conn.data_to_send())
        self.transport.close()

    def on_response_complete(self) -> None:
        self.server_state.total_requests += 1

        if self.transport.is_closing():
            return

        # Set a short Keep-Alive timeout.
        self._unset_keepalive_if_required()

        self.timeout_keep_alive_task = self.loop.call_later(self.timeout_keep_alive, self.timeout_keep_alive_handler)

        # Unpause data reads if needed.
        self.flow.resume_reading()

        # Do not reset the parser until the complete request body has arrived.
        # An application may respond before consuming it; any remaining body
        # still belongs to the current request and must be discarded as such.
        if self.cycle is not None and not self.cycle.more_body:
            self.conn.start_next_cycle()
            self.handle_events()

    def shutdown(self) -> None:
        """
        Called by the server to commence a graceful shutdown.
        """
        if self.cycle is None or self.cycle.response_complete:
            self.transport.close()
        else:
            self.cycle.keep_alive = False

    def pause_writing(self) -> None:
        """
        Called by the transport when the write buffer exceeds the high water mark.
        """
        self.flow.pause_writing()  # pragma: no cover

    def resume_writing(self) -> None:
        """
        Called by the transport when the write buffer drops below the low water mark.
        """
        self.flow.resume_writing()  # pragma: no cover

    def timeout_keep_alive_handler(self) -> None:
        """
        Called on a keep-alive connection if no new data is received after a short
        delay.
        """
        if not self.transport.is_closing():
            self.transport.close()


class RequestResponseCycle:
    def __init__(
        self,
        scope: HTTPScope,
        conn: zttp.H1Connection,
        transport: asyncio.Transport,
        flow: FlowControl,
        logger: logging.Logger,
        access_logger: logging.Logger,
        access_log: bool,
        default_headers: list[tuple[bytes, bytes]],
        message_event: asyncio.Event,
        expect_100_continue: bool,
        on_response: Callable[..., None],
    ) -> None:
        self.scope = scope
        self.conn = conn
        self.transport = transport
        self.flow = flow
        self.logger = logger
        self.access_logger = access_logger
        self.access_log = access_log
        self.default_headers = default_headers
        self.message_event = message_event
        self.on_response = on_response

        # Connection state
        self.disconnected = False
        self.keep_alive = True
        self.waiting_for_100_continue = expect_100_continue

        # Request state
        self.body = bytearray()
        self.more_body = True

        # Response state
        self.response_started = False
        self.response_complete = False
        self.bodyless = False
        self.chunked_encoding = False
        self.expected_content_length = 0

    # ASGI exception wrapper
    async def run_asgi(self, app: ASGI3Application) -> None:
        try:
            result = await app(  # type: ignore[func-returns-value]
                self.scope, self.receive, self.send
            )
        except BaseException as exc:
            msg = "Exception in ASGI application\n"
            self.logger.error(msg, exc_info=exc)
            if not self.response_started:
                await self.send_500_response()
            else:
                self.transport.close()
        else:
            if result is not None:
                msg = "ASGI callable should return None, but returned '%s'."
                self.logger.error(msg, result)
                self.transport.close()
            elif not self.response_started and not self.disconnected:
                msg = "ASGI callable returned without starting response."
                self.logger.error(msg)
                await self.send_500_response()
            elif not self.response_complete and not self.disconnected:
                msg = "ASGI callable returned without completing response."
                self.logger.error(msg)
                self.transport.close()
        finally:
            self.on_response = lambda: None

    async def send_500_response(self) -> None:
        response_start_event: HTTPResponseStartEvent = {
            "type": "http.response.start",
            "status": 500,
            "headers": [
                (b"content-type", b"text/plain; charset=utf-8"),
                (b"connection", b"close"),
            ],
        }
        await self.send(response_start_event)
        response_body_event: HTTPResponseBodyEvent = {
            "type": "http.response.body",
            "body": b"Internal Server Error",
            "more_body": False,
        }
        await self.send(response_body_event)

    # ASGI interface
    async def send(self, message: ASGISendEvent) -> None:
        if self.flow.write_paused and not self.disconnected:
            await self.flow.drain()  # pragma: no cover

        if self.disconnected:
            return  # pragma: no cover

        if not self.response_started:
            # Sending response status line and headers
            if message["type"] != "http.response.start":
                raise RuntimeError(f"Expected ASGI message 'http.response.start', but got '{message['type']}'.")

            self.response_started = True
            self.waiting_for_100_continue = False

            status = message["status"]
            headers = self.default_headers + list(message.get("headers", []))

            if CLOSE_HEADER in self.scope["headers"] and CLOSE_HEADER not in headers:
                headers = headers + [CLOSE_HEADER]

            bodyless = self.scope["method"] == "HEAD" or status in (204, 304) or status < 200

            # RFC 9112 §6.1 forbids Transfer-Encoding on 1xx and 204 responses, and
            # zttp refuses to serialize one there, so drop it instead of erroring.
            # HEAD and 304 may keep it: it describes the body a GET would return.
            if status < 200 or status == 204:
                headers = [(name, value) for name, value in headers if name.lower() != b"transfer-encoding"]

            has_content_length = False
            has_transfer_encoding = False
            for name, value in headers:
                name = name.lower()
                if name == b"content-length":
                    has_content_length = True
                    self.expected_content_length = int(value.decode())
                elif name == b"transfer-encoding":
                    # zttp only accepts a sole, final `chunked` coding - anything
                    # else raises, like h11 - and it frames the body itself, so
                    # there is no Content-Length accounting to do here.
                    has_transfer_encoding = True
                    self.chunked_encoding = True

            # A response carrying both Content-Length and Transfer-Encoding is
            # a framing conflict that zttp rejects, so drop the Content-Length.
            if has_transfer_encoding and has_content_length:
                headers = [(name, value) for name, value in headers if name.lower() != b"content-length"]
                has_content_length = False
                self.expected_content_length = 0

            # zttp refuses to frame the body unless the response declares
            # Content-Length or Transfer-Encoding, so add chunked encoding
            # ourselves when the application provides neither.
            if not bodyless and not has_transfer_encoding and not has_content_length:
                self.chunked_encoding = True
                headers = headers + [(b"transfer-encoding", b"chunked")]

            if self.access_log:
                self.access_logger.info(
                    '%s - "%s %s HTTP/%s" %d',
                    get_client_addr(self.scope),
                    self.scope["method"],
                    get_path_with_query_string(self.scope),
                    self.scope["http_version"],
                    status,
                )

            # Write response status line and headers
            self.bodyless = bodyless
            self.conn.send_response(status, headers)
            self.transport.write(self.conn.data_to_send())

        elif not self.response_complete:
            # Sending response body
            if message["type"] != "http.response.body":
                raise RuntimeError(f"Expected ASGI message 'http.response.body', but got '{message['type']}'.")

            body = message.get("body", b"")
            more_body = message.get("more_body", False)

            # Write response body
            if self.bodyless:
                self.expected_content_length = 0
            elif self.chunked_encoding:
                if body:
                    self.conn.send_data(body)
                    self.transport.write(self.conn.data_to_send())
            else:
                if len(body) > self.expected_content_length:
                    raise RuntimeError("Response content longer than Content-Length")
                self.expected_content_length -= len(body)
                if body:
                    self.conn.send_data(body)
                    self.transport.write(self.conn.data_to_send())

            # Handle response completion
            if not more_body:
                if self.expected_content_length != 0:
                    raise RuntimeError("Response content shorter than Content-Length")
                self.response_complete = True
                self.message_event.set()
                self.conn.end_message()
                self.transport.write(self.conn.data_to_send())

        else:
            # Response already sent
            raise RuntimeError(f"Unexpected ASGI message '{message['type']}' sent, after response already completed.")

        if self.response_complete:
            # `should_close()` covers both directions: the request's `Connection: close`
            # or HTTP/1.0 default, and a `close` the response we just wrote declared.
            if not self.keep_alive or self.conn.should_close():
                self.transport.close()
            self.on_response()

    async def receive(self) -> ASGIReceiveEvent:
        if self.waiting_for_100_continue and not self.transport.is_closing():
            self.conn.send_informational(100)
            self.transport.write(self.conn.data_to_send())
            self.waiting_for_100_continue = False

        if not self.disconnected and not self.response_complete:
            self.flow.resume_reading()
            await self.message_event.wait()
            self.message_event.clear()

        if self.disconnected or self.response_complete:
            return {"type": "http.disconnect"}

        message: HTTPRequestEvent = {"type": "http.request", "body": bytes(self.body), "more_body": self.more_body}
        self.body = bytearray()
        return message



"""
    pygments.styles.rrt
    ~~~~~~~~~~~~~~~~~~~

    pygments "rrt" theme, based on Zap and Emacs defaults.

    :copyright: Copyright 2006-present by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from pygments.style import Style
from pygments.token import (
    Comment,
    Generic,
    Keyword,
    Name,
    Number,
    Operator,
    String,
    Token,
)

__all__ = ['RrtStyle']


class RrtStyle(Style):
    """
    Minimalistic "rrt" theme, based on Zap and Emacs defaults.
    """

    name = 'rrt'

    background_color = '#000000'
    highlight_color = '#0000ff'

    styles = {
        Token:              '#dddddd',
        Comment:            '#00ff00',
        Generic.Emph:       '#ffffff',
        Generic.Heading:    '#ffff00',
        Generic.Subheading: '#ffff00',
        Generic.Strong:     '#ffffff',
        Name.Entity:        '#eedd82',
        Name.Function:      '#ffff00',
        Name.Variable:      '#eedd82',
        Name.Tag:           '#7fffd4',
        Name.Constant:      '#7fffd4',
        Keyword:            '#ff0000',
        Operator.Word:      '#ff0000',
        Comment.Preproc:    '#e5e5e5',
        String:             '#87ceeb',
        Keyword.Type:       '#ee82ee',
        Number:             '#ff00ff',
    }



from __future__ import annotations

from collections.abc import Callable, Sequence
from typing import Any, ParamSpec

from starlette._utils import is_async_callable
from starlette.concurrency import run_in_threadpool

P = ParamSpec("P")


class BackgroundTask:
    def __init__(self, func: Callable[P, Any], *args: P.args, **kwargs: P.kwargs) -> None:
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.is_async = is_async_callable(func)

    async def __call__(self) -> None:
        if self.is_async:
            await self.func(*self.args, **self.kwargs)
        else:
            await run_in_threadpool(self.func, *self.args, **self.kwargs)


class BackgroundTasks(BackgroundTask):
    def __init__(self, tasks: Sequence[BackgroundTask] | None = None):
        self.tasks = list(tasks) if tasks else []

    def add_task(self, func: Callable[P, Any], *args: P.args, **kwargs: P.kwargs) -> None:
        task = BackgroundTask(func, *args, **kwargs)
        self.tasks.append(task)

    async def __call__(self) -> None:
        for task in self.tasks:
            await task()
